@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2002-2004 by Gunnar Kedenburg                */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -1071,8 +1070,7 @@ void BmpEncoder::setNumBands( unsigned int bands )
     pimpl->grayscale = ( bands == 1 );
 }
 
-void BmpEncoder::setCompressionType( const std::string & comp,
-                                     int quality )
+void BmpEncoder::setCompressionType( const std::string & /* comp */, int /* quality */)
 {
     VIGRA_IMPEX_FINALIZED(pimpl->finalized);
 }
