@@ -3,9 +3,8 @@
 /*               Copyright 2001-2002 by Pablo d'Angelo                  */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.4.0, Dec 21 2005 )                                    */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -285,8 +284,7 @@ namespace vigra {
         }
     };
 
-    void HDREncoderImpl::setCompressionType( const std::string & comp,
-                                              int quality = -1 )
+    void HDREncoderImpl::setCompressionType( const std::string & /* comp */, int /* quality */)
     {
     }
 
@@ -346,8 +344,7 @@ namespace vigra {
         pimpl->samples_per_pixel = 3;
     }
 
-    void HDREncoder::setCompressionType( const std::string & comp,
-                                          int quality )
+    void HDREncoder::setCompressionType( const std::string & /* comp */, int /* quality */)
     {
     }
 
@@ -359,15 +356,15 @@ namespace vigra {
         pimpl->pixeltype = "FLOAT";
     }
 
-    void HDREncoder::setPosition( const vigra::Diff2D & pos )
+    void HDREncoder::setPosition( const vigra::Diff2D &)
     {
     }
 
-    void HDREncoder::setXResolution( float xres )
+    void HDREncoder::setXResolution( float )
     {
     }
 
-    void HDREncoder::setYResolution( float yres )
+    void HDREncoder::setYResolution( float )
     {
     }
 
