@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2002 by Gunnar Kedenburg                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -59,6 +58,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include "vigra/array_vector.hxx"
 #include "vigra/imageinfo.hxx"
 #include "codecmanager.hxx"
 #include "vigra/multi_impex.hxx"
@@ -87,7 +87,7 @@ bool splitString(
     const std::string &s, char separator, std::string &a, std::string &b,
     bool reverse = false)
 {
-    unsigned int splitPos = (reverse ? s.rfind(separator) : s.find(separator));
+    std::size_t splitPos = (reverse ? s.rfind(separator) : s.find(separator));
     if(splitPos >= s.size())
         return false;
     a = std::string(s.begin(), s.begin() + splitPos);
@@ -100,7 +100,7 @@ std::string trimString(const std::string &s)
     unsigned int begin = 0;
     while(begin < s.size() && ((s[begin] == ' ') || (s[begin] == '\t')))
         ++begin;
-    unsigned int end = s.size();
+    std::size_t end = s.size();
     while(end > 0 && ((s[end-1] == ' ') || (s[end-1] == '\t')))
         --end;
     return std::string(s.begin() + begin, s.begin() + end);
@@ -114,10 +114,10 @@ std::string trimString(const std::string &s)
 void splitPathFromFilename(const std::string &pathAndName,
                            std::string &path, std::string &name)
 {
-	// on Windows, both '/' and '\' are valid path separators
-	// note: std::basic_string.rfind() may return 'unsigned int', so explicitely cast to 'int'
-	int split = std::max(static_cast<int>(pathAndName.rfind('/')), static_cast<int>(pathAndName.rfind('\\')));
-	if(split == static_cast<int>(std::string::npos))
+    // on Windows, both '/' and '\' are valid path separators
+    // note: std::basic_string.rfind() may return 'unsigned int', so explicitely cast to 'int'
+    int split = std::max(static_cast<int>(pathAndName.rfind('/')), static_cast<int>(pathAndName.rfind('\\')));
+    if(split == static_cast<int>(std::string::npos))
     {
         path = ".";
         name = pathAndName;
@@ -146,7 +146,7 @@ VIGRA_EXPORT void findImageSequence(const std::string &name_base,
     WIN32_FIND_DATA FileData;
 
     std::string path, base;
-	splitPathFromFilename(name_base, path, base);
+    splitPathFromFilename(name_base, path, base);
 
     std::vector<std::string> result;
     char numbuf[21], extbuf[1024];
@@ -220,7 +220,7 @@ void findImageSequence(const std::string &name_base,
 {
     // find out how many images we have
     std::string path, base;
-	splitPathFromFilename(name_base, path, base);
+    splitPathFromFilename(name_base, path, base);
 
     DIR * dir = opendir(path.c_str());
     if(!dir)
@@ -349,7 +349,7 @@ ImageExportInfo & ImageExportInfo::setForcedRangeMapping(double fromMin, double
 
 bool ImageExportInfo::hasForcedRangeMapping() const
 {
-    return fromMax_ > fromMin_;
+    return (fromMax_ > fromMin_) || (toMax_ > toMin_);
 }
 
 double ImageExportInfo::getFromMin() const
@@ -528,6 +528,8 @@ ImageImportInfo::ImageImportInfo( const char * filename )
     m_num_bands = decoder->getNumBands();
     m_num_extra_bands = decoder->getNumExtraBands();
     m_pos = decoder->getPosition();
+    m_x_res = decoder->getXResolution();
+    m_y_res = decoder->getYResolution();
 
     m_icc_profile = decoder->getICCProfile();
 
@@ -554,7 +556,7 @@ const char * ImageImportInfo::getPixelType() const
 
 ImageImportInfo::PixelType ImageImportInfo::pixelType() const
 {
-   const std::string pixeltype=getPixelType();
+    const std::string pixeltype=ImageImportInfo::getPixelType();
    if (pixeltype == "UINT8")
      return UINT8;
    if (pixeltype == "INT16")
@@ -598,6 +600,11 @@ Size2D ImageImportInfo::size() const
     return Size2D( m_width, m_height );
 }
 
+MultiArrayShape<2>::type ImageImportInfo::shape() const
+{
+    return MultiArrayShape<2>::type( m_width, m_height );
+}
+
 bool ImageImportInfo::isGrayscale() const
 {
     return m_num_bands == 1;
@@ -641,6 +648,160 @@ std::auto_ptr<Decoder> decoder( const ImageImportInfo & info )
     return getDecoder( std::string( info.getFileName() ), filetype );
 }
 
+// class VolumeExportInfo
+
+VolumeExportInfo::VolumeExportInfo( const char * name_base, const char * name_ext ) :
+        m_x_res(0), m_y_res(0), m_z_res(0),
+        m_filename_base(name_base), m_filename_ext(name_ext),
+        fromMin_(0.0), fromMax_(0.0), toMin_(0.0), toMax_(0.0)
+{
+}
+
+VolumeExportInfo::~VolumeExportInfo()
+{
+}
+
+VolumeExportInfo & VolumeExportInfo::setFileType( const char * filetype )
+{
+    m_filetype = filetype;
+    return *this;
+}
+
+VolumeExportInfo & VolumeExportInfo::setForcedRangeMapping(double fromMin, double fromMax,
+                                                     double toMin, double toMax) 
+{
+    fromMin_ = fromMin;
+    fromMax_ = fromMax;
+    toMin_ = toMin;
+    toMax_ = toMax;
+    return *this;
+}
+
+bool VolumeExportInfo::hasForcedRangeMapping() const
+{
+    return (fromMax_ > fromMin_) || (toMax_ > toMin_);
+}
+
+double VolumeExportInfo::getFromMin() const
+{
+    return fromMin_;
+}
+
+double VolumeExportInfo::getFromMax() const
+{
+    return fromMax_;
+}
+
+double VolumeExportInfo::getToMin() const
+{
+    return toMin_;
+}
+
+double VolumeExportInfo::getToMax() const
+{
+    return toMax_;
+}
+
+VolumeExportInfo & VolumeExportInfo::setCompression( const char * comp )
+{
+    m_comp = comp;
+    return *this;
+}
+
+VolumeExportInfo & VolumeExportInfo::setFileNameExt(const char * name_ext)
+{
+    m_filename_ext = name_ext;
+    return *this;
+}
+
+const char * VolumeExportInfo::getFileNameExt() const
+{
+    return m_filename_ext.c_str();
+}
+
+VolumeExportInfo & VolumeExportInfo::setFileNameBase(const char * name_base)
+{
+    m_filename_base = name_base;
+    return *this;
+}
+
+const char * VolumeExportInfo::getFileNameBase() const
+{
+    return m_filename_base.c_str();
+}
+
+const char * VolumeExportInfo::getFileType() const
+{
+    return m_filetype.c_str();
+}
+
+VolumeExportInfo & VolumeExportInfo::setPixelType( const char * s )
+{
+    m_pixeltype = s;
+    return *this;
+}
+
+const char * VolumeExportInfo::getPixelType() const
+{
+    return m_pixeltype.c_str();
+}
+
+const char * VolumeExportInfo::getCompression() const
+{
+    return m_comp.c_str();
+}
+
+float VolumeExportInfo::getXResolution() const
+{
+    return m_x_res;
+}
+
+float VolumeExportInfo::getYResolution() const
+{
+    return m_y_res;
+}
+
+VolumeExportInfo & VolumeExportInfo::setXResolution( float val )
+{
+    m_x_res = val;
+    return *this;
+}
+
+VolumeExportInfo & VolumeExportInfo::setYResolution( float val )
+{
+    m_y_res = val;
+    return *this;
+}
+
+VolumeExportInfo & VolumeExportInfo::setZResolution( float val )
+{
+    m_z_res = val;
+    return *this;
+}
+
+VolumeExportInfo & VolumeExportInfo::setPosition(const vigra::Diff2D & pos)
+{
+    m_pos = pos;
+    return *this;
+}
+
+vigra::Diff2D VolumeExportInfo::getPosition() const
+{
+    return m_pos;
+}
+
+const VolumeExportInfo::ICCProfile & VolumeExportInfo::getICCProfile() const
+{
+    return m_icc_profile;
+}
+
+VolumeExportInfo & VolumeExportInfo::setICCProfile(
+    const VolumeExportInfo::ICCProfile &profile)
+{
+    m_icc_profile = profile;
+    return *this;
+}
+
 VolumeImportInfo::VolumeImportInfo(const std::string &filename)
 : shape_(0, 0, 0),
   resolution_(1.f, 1.f, 1.f),
@@ -652,8 +813,8 @@ VolumeImportInfo::VolumeImportInfo(const std::string &filename)
 
     do
     {
-        numEndIt = std::find_if(numBeginIt, filename.rend(), &isdigit);
-        numBeginIt = std::find_if(numEndIt, filename.rend(), not1(std::ptr_fun(&isdigit)));
+        numEndIt = std::find_if(numBeginIt, filename.rend(),(int (*)(int)) &isdigit);
+        numBeginIt = std::find_if(numEndIt, filename.rend(), not1(std::ptr_fun((int (*)(int))&isdigit)));
 
         if(numEndIt != filename.rend())
         {
@@ -793,4 +954,39 @@ void VolumeImportInfo::getVolumeInfoFromFirstSlice(const std::string &filename)
     numBands_ = info.numBands();
 }
 
+VolumeImportInfo::ShapeType VolumeImportInfo::shape() const { return shape_; }
+VolumeImportInfo::Resolution VolumeImportInfo::resolution() const { return resolution_; }
+VolumeImportInfo::PixelType VolumeImportInfo::pixelType() const
+{
+    const std::string pixeltype=VolumeImportInfo::getPixelType();
+   if (pixeltype == "UINT8")
+       return ImageImportInfo::UINT8;
+   if (pixeltype == "INT16")
+     return ImageImportInfo::INT16;
+   if (pixeltype == "UINT16")
+     return ImageImportInfo::UINT16;
+   if (pixeltype == "INT32")
+     return ImageImportInfo::INT32;
+   if (pixeltype == "UINT32")
+     return ImageImportInfo::UINT32;
+   if (pixeltype == "FLOAT")
+     return ImageImportInfo::FLOAT;
+   if (pixeltype == "DOUBLE")
+     return ImageImportInfo::DOUBLE;
+   vigra_fail( "internal error: unknown pixel type" );
+   return VolumeImportInfo::PixelType();
+}
+const char * VolumeImportInfo::getPixelType() const
+{
+    return pixelType_.c_str();
+}
+MultiArrayIndex VolumeImportInfo::numBands() const { return numBands_; }
+bool VolumeImportInfo::isGrayscale() const { return numBands_ == 1; }
+bool VolumeImportInfo::isColor() const { return numBands_ > 1; }
+MultiArrayIndex VolumeImportInfo::width() const { return shape_[0]; }
+MultiArrayIndex VolumeImportInfo::height() const { return shape_[1]; }
+MultiArrayIndex VolumeImportInfo::depth() const { return shape_[2]; }
+const std::string & VolumeImportInfo::name() const { return name_; }
+const std::string & VolumeImportInfo::description() const { return description_; }
+
 } // namespace vigra
