@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2001-2002 by Gunnar Kedenburg                */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -198,9 +197,9 @@ namespace vigra {
         // check if the file is a png file
         const unsigned int sig_size = 8;
         png_byte sig[sig_size];
-        std::fread( sig, sig_size, 1, file.get() );
+        std::size_t readCount = std::fread( sig, sig_size, 1, file.get() );
         const int no_png = png_sig_cmp( sig, 0, sig_size );
-        vigra_precondition( !no_png, "given file is not a png file.");
+        vigra_precondition( (readCount == 1) && !no_png, "given file is not a png file.");
 
         // create png read struct with user defined handlers
         png = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL,
@@ -309,8 +308,8 @@ namespace vigra {
         }
 
         // read resolution
-        x_resolution = png_get_x_pixels_per_meter( png, info ) / 254.0;
-        y_resolution = png_get_y_pixels_per_meter( png, info ) / 254.0;
+        x_resolution = png_get_x_pixels_per_meter( png, info ) * 0.0254f;
+        y_resolution = png_get_y_pixels_per_meter( png, info ) * 0.0254f;
 
         // read offset
         position.x = png_get_x_offset_pixels( png, info );
@@ -576,8 +575,8 @@ namespace vigra {
         if (x_resolution > 0 && y_resolution > 0) {
             if (setjmp(png->jmpbuf))
                 vigra_postcondition( false, png_error_message.insert(0, "error in png_set_pHYs(): ").c_str() );
-            png_set_pHYs(png, info, (png_uint_32) (x_resolution * 254 + 0.5),
-                         (png_uint_32) (y_resolution * 254 + 0.5),
+            png_set_pHYs(png, info, (png_uint_32) (x_resolution / 0.0254 + 0.5),
+                         (png_uint_32) (y_resolution / 0.0254 + 0.5),
                          PNG_RESOLUTION_METER);
         }
 
@@ -591,8 +590,8 @@ namespace vigra {
 #if (PNG_LIBPNG_VER > 10008) && defined(PNG_WRITE_iCCP_SUPPORTED)
         // set icc profile
         if (iccProfile.size() > 0) {
-            png_set_iCCP(png, info, "icc", 0,
-                         (char *)iccProfile.begin(), iccProfile.size());
+            png_set_iCCP(png, info, const_cast<char*>("icc"), 0,
+                         (char *)iccProfile.begin(), (png_uint_32)iccProfile.size());
         }
 #endif
 
@@ -680,8 +679,7 @@ namespace vigra {
         pimpl->components = bands;
     }
 
-    void PngEncoder::setCompressionType( const std::string & comp,
-                                         int quality )
+    void PngEncoder::setCompressionType( const std::string & /* comp */, int /* quality */)
     {
         // nothing is settable => do nothing
     }
