@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2002-2004 by Gunnar Kedenburg                */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -234,7 +233,7 @@ namespace vigra {
     {
         // cast the bands to the correct type
         typedef void_vector< UInt8 > vector_type;
-        vector_type & cbands = reinterpret_cast< vector_type & >(bands);
+        vector_type & cbands = static_cast< vector_type & >(bands);
 
         // read and store
         stream.read( reinterpret_cast< char * >(cbands.data()),
@@ -245,7 +244,7 @@ namespace vigra {
     {
         // cast the bands to the correct type
         typedef void_vector< UInt16 > vector_type;
-        vector_type & cbands = reinterpret_cast< vector_type & >(bands);
+        vector_type & cbands = static_cast< vector_type & >(bands);
 
         // read and store, need to swap bytes.
         byteorder bo( "big endian" );
@@ -257,7 +256,7 @@ namespace vigra {
     {
         // cast the bands to the correct type
         typedef void_vector< UInt32 > vector_type;
-        vector_type & cbands = reinterpret_cast< vector_type & >(bands);
+        vector_type & cbands = static_cast< vector_type & >(bands);
 
         byteorder bo( "big endian" );
         read_array( stream, bo, reinterpret_cast< UInt32 * >(cbands.data()),
@@ -565,8 +564,7 @@ namespace vigra {
         pimpl->components = bands;
     }
 
-    void PnmEncoder::setCompressionType( const std::string & comp,
-                                         int quality )
+    void PnmEncoder::setCompressionType( const std::string & comp, int /* quality */)
     {
         VIGRA_IMPEX_FINALIZED(pimpl->finalized);
         if ( comp == "ASCII" )
