@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*               Copyright 2002 by Gunnar Kedenburg                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -261,6 +260,9 @@ namespace vigra {
                         const void_vector<map_storage_type> & maps,
                         unsigned int map_bands, unsigned int map_width, unsigned int map_height )
     {
+        vigra_precondition(src_bands == 1u,
+               "map_multiband(): Source image must have one band.");
+        
         typedef colormap< storage_type, map_storage_type > colormap_type;
         const unsigned int num_pixels = src_width * src_height;
 
@@ -362,13 +364,13 @@ namespace vigra {
         vigra_precondition( stream.get() == XV_IMAGE_VER_NUM,
                             "file format version unsupported" );
 
-        // check the endianess
+        // check the endianness
         const char machine_dep = stream.get();
         if ( machine_dep == VFF_DEP_BIGENDIAN )
             bo.set("big endian");
         else if ( machine_dep == VFF_DEP_LITENDIAN )
             bo.set("little endian");
-        else vigra_fail( "endianess unsupported" );
+        else vigra_fail( "endianness unsupported" );
 
         // skip the comments
         stream.seekg( 0x208, std::ios::beg );
@@ -954,8 +956,7 @@ namespace vigra {
         pimpl->header.num_data_bands = static_cast<ViffHeader::field_type>(numBands);
     }
 
-    void ViffEncoder::setCompressionType( const std::string & comp,
-                                          int quality )
+    void ViffEncoder::setCompressionType( const std::string & /* comp */, int /* quality */)
     {
         VIGRA_IMPEX_FINALIZED(pimpl->finalized);
     }
