@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -36,6 +35,7 @@
 
 #include <iostream>
 #include <cmath>
+#include <cstdio>
 #include "vigra/convolution.hxx"
 #include "unittest.hxx"
 #include "vigra/stdimage.hxx"
@@ -579,10 +579,20 @@ struct ConvolutionTest
     void separableConvolutionTest()
     {
         vigra::Kernel1D<double> binom;
-        binom.initBinomial(1);
+        binom.initBinomial(2);
         
         vigra::Kernel1D<double>::Iterator center = binom.center();
         
+        should(center[0] == 0.375);
+        should(center[-1] == 0.25);
+        should(center[1] == 0.25);
+        should(center[-2] == 0.0625);
+        should(center[2] == 0.0625);
+        
+        binom.initBinomial(1);
+        
+        center = binom.center();
+        
         should(center[0] == 0.5);
         should(center[-1] == 0.25);
         should(center[1] == 0.25);
@@ -2134,11 +2144,11 @@ struct ConvolutionTestSuite
     }
 };
 
-int main()
+int main(int argc, char ** argv)
 {
     ConvolutionTestSuite test;
 
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << test.report() << std::endl;
 
