@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -81,15 +80,15 @@ RGBValue<float> * testData(RGBValue<float>)
 {
     typedef vigra::RGBValue<float> FRGB;
     static FRGB data[] = {
-        FRGB(1.1, 1.1, 1.1),
-        FRGB(2.2, 2.2, 2.2),
-        FRGB(3.3, 3.3, 3.3),
-        FRGB(4.4, 4.4, 4.4),
-        FRGB(5.5, 5.5, 5.5),
-        FRGB(6.6, 6.6, 6.6),
-        FRGB(7.7, 7.7, 7.7),
-        FRGB(8.8, 8.8, 8.8),
-        FRGB(9.9, 9.9, 9.9)
+        FRGB(1.1f, 1.1f, 1.1f),
+        FRGB(2.2f, 2.2f, 2.2f),
+        FRGB(3.3f, 3.3f, 3.3f),
+        FRGB(4.4f, 4.4f, 4.4f),
+        FRGB(5.5f, 5.5f, 5.5f),
+        FRGB(6.6f, 6.6f, 6.6f),
+        FRGB(7.7f, 7.7f, 7.7f),
+        FRGB(8.8f, 8.8f, 8.8f),
+        FRGB(9.9f, 9.9f, 9.9f)
     };
     return data;
 }
@@ -102,8 +101,9 @@ struct ImageTest
     value_type internalMemory[9];
     value_type * data;
 
-    ImageTest(IMAGE const & image)
-    : img(image), data(testData(value_type()))
+    ImageTest(IMAGE const & image) :
+            data(testData(value_type())),
+            img(image)
     {
         typename Image::Accessor acc = img.accessor();
         typename Image::iterator i = img.begin();
@@ -267,7 +267,14 @@ struct ImageTest
     {
         Image img1(img.width(), img.height(), data);
         
+        shouldEqual(img1.width(), img.width());
+        shouldEqual(img1.height(), img.height());
         shouldEqualSequence(img.begin(), img.end(), img1.begin());
+        
+        Image img2(img.width(), img.height(), SkipInitialization);
+        
+        shouldEqual(img2.width(), img.width());
+        shouldEqual(img2.height(), img.height());
     }
 
     void copyImage()
@@ -284,10 +291,10 @@ struct ImageTest
             should(acc(i) == acc(i1));
         }
 
-        img.init(0);
+        img.init(NumericTraits<Value>::zero());
         for(; i != img.end(); ++i)
         {
-			should(acc(i) == Value(0));
+			should(acc(i) == NumericTraits<Value>::zero());
         }
         img(1,1) = Value(200);
         img1 = img;
@@ -538,7 +545,7 @@ struct ImageContainerTests
 		shouldEqual(ia2.size(), ia.size()-1);
 
 		ia.clear();
-		shouldEqual(ia.size(), 0);
+		shouldEqual(ia.size(), 0u);
 	}
 };
 
@@ -895,11 +902,11 @@ struct ImageTestCollection
    }
 };
 
-int main()
+int main(int argc, char ** argv)
 {
     ImageTestCollection test;
 
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << test.report() << std::endl;
 
