@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -45,6 +44,7 @@
 #include "vigra/basicgeometry.hxx"
 #include "vigra/affinegeometry.hxx"
 #include "vigra/impex.hxx"
+#include "vigra/meshgrid.hxx"
 
 using namespace vigra;
 
@@ -394,6 +394,32 @@ struct ImageFunctionsTest
         shouldEqual(f(), 49.5);
     }
 
+    void meshGridTest()
+    {
+        // create an image whose values are equal to each pixel's distance from the image center
+        int width = 5, height = 7;
+        int xc = width/2, yc = height/2;
+
+        DImage dist(width, height);
+        Point2D upperLeft(-xc, -yc);
+
+        using namespace vigra::functor;
+        transformImage(meshGrid(upperLeft, upperLeft+dist.size()),
+                       destImage(dist),
+                       norm(Arg1()));
+        for(int y=0; y<height; ++y)
+            for(int x=0; x<width; ++x)
+                shouldEqualTolerance(dist(x,y), vigra::hypot(x-xc, y-yc), 1e-14);
+        dist.init(0.0);
+        transformImage(meshGrid(Rect2D(upperLeft, upperLeft+dist.size())),
+                       destImage(dist),
+                       norm(Arg1()));
+        for(int y=0; y<height; ++y)
+            for(int x=0; x<width; ++x)
+                shouldEqualTolerance(dist(x,y), vigra::hypot(x-xc, y-yc), 1e-14);
+    }
+
+
     void findBoundingRectangleTest()
     {
         vigra::FindBoundingRectangle findRect;
@@ -660,13 +686,10 @@ struct ImageFunctionsTest
         BImage img1(3,3);
 
         transformImage(srcImageRange(img), destImage(img1),
-					   linearRangeMapping(1.0, 10.0, 0, 255));
+					   linearRangeMapping(1.0, 10.0, 0, 250));
+
+        int res[] = {3, 33, 64, 94, 125, 156, 186, 217, 247 };
 
-#if defined(__i386__)
-        int res[] = {3, 34, 65, 96, 127, 159, 190, 221, 252 };
-#else
-        int res[] = {3, 34, 65, 96, 128, 159, 190, 221, 252 };
-#endif
         shouldEqualSequence(img1.begin(), img1.end(), res);
 
         BRGBImage img2(3,3);
@@ -752,7 +775,7 @@ struct ImageFunctionsTest
 
         gradientBasedTransform(
           srcImageRange(in, vigra::RedAccessor<RGBValue >()),
-          destImage(res), vigra::MagnitudeFunctor<float>());
+          destImage(res), vigra::MagnitudeFunctor<double>());
 
         for(y=0; y<3; ++y)
         {
@@ -764,7 +787,7 @@ struct ImageFunctionsTest
 
         gradientBasedTransform(
           srcImageRange(in),
-          destImage(res), vigra::RGBGradientMagnitudeFunctor<float>());
+          destImage(res), vigra::RGBGradientMagnitudeFunctor<double>());
 
         for(y=0; y<3; ++y)
         {
@@ -825,15 +848,13 @@ struct ImageFunctionsTest
 
     void resizeNoInterpolationTest()
     {
-        Image img1(5,5);
+        Image img1(5,5), img3(5,5);
 
         resizeImageNoInterpolation(srcImageRange(img), destImageRange(img1));
 
         Image::ScanOrderIterator i1 = img1.begin();
         Image::Accessor acc = img.accessor();
 
-        should(acc(i1) == 1.1);
-        ++i1;
         should(acc(i1) == 1.1);
         ++i1;
         should(acc(i1) == 2.2);
@@ -842,27 +863,17 @@ struct ImageFunctionsTest
         ++i1;
         should(acc(i1) == 3.3);
         ++i1;
-        should(acc(i1) == 1.1);
-        ++i1;
-        should(acc(i1) == 1.1);
-        ++i1;
-        should(acc(i1) == 2.2);
-        ++i1;
-        should(acc(i1) == 2.2);
-        ++i1;
         should(acc(i1) == 3.3);
         ++i1;
         should(acc(i1) == 4.4);
         ++i1;
-        should(acc(i1) == 4.4);
-        ++i1;
         should(acc(i1) == 5.5);
         ++i1;
         should(acc(i1) == 5.5);
         ++i1;
         should(acc(i1) == 6.6);
         ++i1;
-        should(acc(i1) == 4.4);
+        should(acc(i1) == 6.6);
         ++i1;
         should(acc(i1) == 4.4);
         ++i1;
@@ -872,7 +883,7 @@ struct ImageFunctionsTest
         ++i1;
         should(acc(i1) == 6.6);
         ++i1;
-        should(acc(i1) == 7.7);
+        should(acc(i1) == 6.6);
         ++i1;
         should(acc(i1) == 7.7);
         ++i1;
@@ -881,6 +892,18 @@ struct ImageFunctionsTest
         should(acc(i1) == 8.8);
         ++i1;
         should(acc(i1) == 9.9);
+        ++i1;
+        should(acc(i1) == 9.9);
+        ++i1;
+        should(acc(i1) == 7.7);
+        ++i1;
+        should(acc(i1) == 8.8);
+        ++i1;
+        should(acc(i1) == 8.8);
+        ++i1;
+        should(acc(i1) == 9.9);
+        ++i1;
+        should(acc(i1) == 9.9);
 
         Image img2(3,3);
         img2 = 0;
@@ -894,6 +917,9 @@ struct ImageFunctionsTest
         {
             should(acc(i) == acc(i2));
         }
+
+        resizeImageSplineInterpolation(srcImageRange(img), destImageRange(img3), BSpline<0>());
+        shouldEqualSequence(img1.begin(), img1.end(), img3.begin());
     }
 
     void resizeLinearInterpolationTest()
@@ -1342,14 +1368,14 @@ struct SplineImageViewTest
         shouldEqual(view.image().width(), 0);
         shouldEqual(viewi.image().width(), img.width());
 
-        for(int y=0; y<reference.height()-1; ++y)
+        for(int y=0; y<reference.height(); ++y)
         {
-            for(int x=0; x<reference.width()-1; ++x)
+            for(int x=0; x<reference.width(); ++x)
             {
                 double dx = (double)x / 2.0;
                 double dy = (double)y / 2.0;
-                shouldEqualTolerance(view(dx, dy), reference(x+1, y+1), 1e-4);
-                shouldEqualTolerance(view.unchecked(dx, dy, 0, 0), reference(x+1, y+1), 1e-4);
+                shouldEqualTolerance(view(dx, dy), reference(x, y), 1e-4);
+                shouldEqualTolerance(view.unchecked(dx, dy, 0, 0), reference(x, y), 1e-4);
                 FixedPoint<10, 1> fx(dx), fy(dy);
                 shouldEqual(viewi.unchecked(fx, fy, 0, 0), viewi.unchecked(dx, dy, 0, 0));
             }
@@ -1430,6 +1456,15 @@ struct SplineImageViewTest
         }
         catch(vigra::PreconditionViolation) {}
     }
+
+    void testVectorSIV()
+    {
+        // (compile-time only test for now)
+        BRGBImage rgb(20, 10);
+        SplineImageView<N, TinyVector<float, 3> > view(srcImageRange(rgb));
+        (void)view(4.5, 1.3);
+    }
+
 };
 
 struct GeometricTransformsTest
@@ -1488,6 +1523,14 @@ struct GeometricTransformsTest
         for(int y = 0; y < 10; ++y)
             for(int x = 0; x < 10; ++x)
                 shouldEqual(img(x,y), res1(w-x-1, y));
+
+        double xfactor = 3.0;
+        double yfactor = 2.0;
+        Image res3((int)(w*xfactor), (int)(h*yfactor));
+        resampleImage(srcImageRange(img), destImage(res3), xfactor, yfactor);
+        for(int y = 0; y < 10; ++y)
+            for(int x = 0; x < 10; ++x)
+                shouldEqual(img(x,y), res3(int(x*xfactor), int(y*yfactor)));
     }
 
     void testAffineMatrix()
@@ -1580,6 +1623,7 @@ struct ImageFunctionsTestSuite
         add( testCase( &ImageFunctionsTest::findAverageWeightedTest));
         add( testCase( &ImageFunctionsTest::findAverageAndVarianceTest));
         add( testCase( &ImageFunctionsTest::reduceFunctorTest));
+        add( testCase( &ImageFunctionsTest::meshGridTest));
         add( testCase( &ImageFunctionsTest::findBoundingRectangleTest));
         add( testCase( &ImageFunctionsTest::lastValueFunctorTest));
         add( testCase( &ImageFunctionsTest::arrayOfRegionStatisticsTest));
@@ -1627,6 +1671,7 @@ struct ImageFunctionsTestSuite
         add( testCase( &SplineImageViewTest<5>::testCoefficientArray));
         add( testCase( &SplineImageViewTest<5>::testImageResize));
         add( testCase( &SplineImageViewTest<5>::testOutside));
+        add( testCase( &SplineImageViewTest<5>::testVectorSIV));
 
         add( testCase( &GeometricTransformsTest::testSimpleGeometry));
         add( testCase( &GeometricTransformsTest::testAffineMatrix));
@@ -1635,11 +1680,11 @@ struct ImageFunctionsTestSuite
     }
 };
 
-int main()
+int main(int argc, char ** argv)
 {
     ImageFunctionsTestSuite test;
 
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << test.report() << std::endl;
 
