@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -39,8 +38,14 @@
 #include "vigra/multi_impex.hxx"
 #include "vigra/basicimageview.hxx"
 #include "vigra/navigator.hxx"
+#include "vigra/multi_pointoperators.hxx"
+#include "vigra/tensorutilities.hxx"
+#include "vigra/multi_tensorutilities.hxx"
+#include "vigra/functorexpression.hxx"
+#include "vigra/random.hxx"
 
 using namespace vigra;
+using namespace vigra::functor;
 
 class MultiArrayDataTest
 {
@@ -50,7 +55,7 @@ public:
     // the image is filled from start to beginning (memory-wise) with
     // ascending numbers.
 
-    typedef unsigned int scalar_type;
+    typedef int scalar_type;
     typedef MultiArray <2, scalar_type> array2_type;
     typedef MultiArray <3, scalar_type> array3_type;
     typedef MultiArrayView <3, scalar_type> array3_view_type;
@@ -64,10 +69,17 @@ public:
         : shape3 (10, 10, 10), array3 (shape3, 1)
     {
         // initialize the array to the test data
-        for (unsigned int i = 0; i < 1000; ++i)
+        for (int i = 0; i < 1000; ++i)
             array3.data () [i] = i;
     }
-    
+
+    void testHasData ()
+    {
+        should(array3.hasData());
+        array3_type empty;
+        should(!empty.hasData());
+    }
+
     void testEquality ()
     {
         typedef difference3_type Shape;
@@ -141,6 +153,24 @@ public:
         shouldEqual ((array (0, 1)), 104);
     }
 
+    void test_singletonDimension ()
+    {
+        MultiArrayView <4, scalar_type, UnstridedArrayTag> a0 = array3.insertSingletonDimension(0);
+        shouldEqual ((a0 [TinyVector <int, 4> (0, 4, 0, 0)]), 4);
+        shouldEqual ((a0 [TinyVector <int, 4> (0, 4, 1, 0)]), 14);
+        shouldEqual ((a0 [TinyVector <int, 4> (0, 4, 0, 1)]), 104);
+
+        MultiArrayView <4, scalar_type, UnstridedArrayTag> a1 = array3.insertSingletonDimension(1);
+        shouldEqual ((a1 [TinyVector <int, 4> (4, 0, 0, 0)]), 4);
+        shouldEqual ((a1 [TinyVector <int, 4> (4, 0, 1, 0)]), 14);
+        shouldEqual ((a1 [TinyVector <int, 4> (4, 0, 0, 1)]), 104);
+
+        MultiArrayView <4, scalar_type, UnstridedArrayTag> a3 = array3.insertSingletonDimension(3);
+        shouldEqual ((a3 [TinyVector <int, 4> (4, 0, 0, 0)]), 4);
+        shouldEqual ((a3 [TinyVector <int, 4> (4, 1, 0, 0)]), 14);
+        shouldEqual ((a3 [TinyVector <int, 4> (4, 0, 1, 0)]), 104);
+    }
+
     // subarray tests
     void test_subarray ()
     {
@@ -181,7 +211,7 @@ public:
         shouldEqual (array (1,1,1), 222);
     }
 
-    // permuteIndices tests
+    // permute and transpose tests
     void testPermute ()
     {   
         array3.reshape(difference3_type(3,5,7));
@@ -195,9 +225,25 @@ public:
                     ref(m,k,l) = array3(k,l,m);
 
         MultiArrayView <3, scalar_type, StridedArrayTag>
-                array = array3.permuteDimensions (difference3_type (2, 0, 1));        
-        shouldEqual(ref.shape(), array.shape());
-        should(ref == array);
+                parray = array3.permuteDimensions (difference3_type (2, 0, 1));        
+        shouldEqual(ref.shape(), parray.shape());
+        should(ref == parray);
+
+        MultiArrayView <3, scalar_type, StridedArrayTag>
+                parray_ascending = parray.permuteStridesAscending();        
+        shouldEqual(array3.shape(), parray_ascending.shape());
+        should(array3 == parray_ascending);
+
+        array3_type ref_descending(difference3_type(array3.shape(2), array3.shape(1), array3.shape(0)));
+        for(int k=0; k<array3.shape(0); ++k)
+            for(int l=0; l<array3.shape(1); ++l)
+                for(int m=0; m<array3.shape(2); ++m)
+                    ref_descending(m,l,k) = array3(k,l,m);
+
+        MultiArrayView <3, scalar_type, StridedArrayTag>
+                parray_descending = array3.permuteStridesDescending();        
+        shouldEqual(ref_descending.shape(), parray_descending.shape());
+        should(ref_descending == parray_descending);
 
         array2_type ref2(difference2_type(array3.shape(1), array3.shape(0)));
         for(int k=0; k<array3.shape(0); ++k)
@@ -237,7 +283,7 @@ public:
         shouldEqual(array3.subarray(first, last).norm(2, false), 0.0);
 
         shouldEqual(array3.squaredNorm(), squaredNorm(array3));
-        shouldEqual(array3.norm(), norm(array3));
+        shouldEqual(array3.norm(), vigra::norm(array3));
     }
     
     void testScanOrderAccess()
@@ -571,6 +617,14 @@ public:
         shouldEqual (a3.shape (2), 50);
         shouldEqual(a3(0,0), 0);
     }
+	
+	void test_copy_int_float()
+	{
+		MultiArray<2, float> a(MultiArrayShape<2>::type(2,2));
+		a.init(3);
+		MultiArray<2, unsigned short> b(a.shape());
+		b = a;
+	}
 
     void test_subarray ()
     {
@@ -736,11 +790,18 @@ struct MultiImpexTest
     
     void testImpex()
     {
-        exportVolume(array, std::string("test"), std::string(".tif"));
+#if defined(HasTIFF)
+        const char * ext1 = ".tif";
+#else
+        const char * ext1 = ".xv";
+#endif
+        exportVolume(array, VolumeExportInfo("test", ext1));
         
         Array result;
         
-        importVolume(result, std::string("test"), std::string(".tif"));
+        VolumeImportInfo import_info("test", ext1);
+        result.reshape(import_info.shape());
+        importVolume(import_info, result);
         
         shouldEqual(result.shape(), Shape(2,3,4));
         shouldEqual(result(0,1,0), 1);
@@ -748,9 +809,14 @@ struct MultiImpexTest
         shouldEqual(result(0,1,2), 3);
         shouldEqual(result(0,1,3), 4);
 
-        exportVolume(array, std::string("impex/test"), std::string(".png"));
+#if defined(HasPNG)
+        const char * ext2 = ".png";
+#else
+        const char * ext2 = ".pnm";
+#endif
+        exportVolume(array, std::string("impex/test"), std::string(ext2));
         
-        importVolume(result, std::string("impex/test"), std::string(".png"));
+        importVolume(result, std::string("impex/test"), std::string(ext2));
         
         shouldEqual(result.shape(), Shape(2,3,4));
         shouldEqual(result(0,1,0), 1);
@@ -759,9 +825,9 @@ struct MultiImpexTest
         shouldEqual(result(0,1,3), 4);
 
 #ifdef _WIN32
-        exportVolume(array, std::string("impex\\test"), std::string(".png"));
+        exportVolume(array, VolumeExportInfo("impex\\test", ext2));
         
-        importVolume(result, std::string("impex\\test"), std::string(".png"));
+        importVolume(result, std::string("impex\\test"), std::string(ext2));
         
         shouldEqual(result.shape(), Shape(2,3,4));
         shouldEqual(result(0,1,0), 1);
@@ -773,14 +839,14 @@ struct MultiImpexTest
 };
 
 template <class IMAGE>
-struct ImageTest
+struct ImageViewTest
 {
     typedef typename IMAGE::value_type value_type;
     typedef MultiArray<2, value_type> MA;
     typedef BasicImageView<value_type> Image;
     static typename Image::value_type data[];
 
-    ImageTest()
+    ImageViewTest()
     : ma(TinyVector<int, 2>(3,3)),
       img(makeBasicImageView(ma))
     {
@@ -952,10 +1018,10 @@ struct ImageTest
             should(acc(i) == acc(i1));
         }
 
-        img.init(0);
+        img.init(NumericTraits<Value>::zero());
         for(; i != img.end(); ++i)
         {
-            should(acc(i) == Value(0));
+            should(acc(i) == Value(NumericTraits<Value>::zero()));
         }
         img(1,1) = Value(200);
         img1 = img;
@@ -972,20 +1038,20 @@ struct ImageTest
     Image img;
 };
 
-typedef ImageTest<vigra::BImage> BImageTest;
+typedef ImageViewTest<vigra::BImage> BImageViewTest;
 
 template <>
-unsigned char BImageTest::data[] = {1,2,3,4,5,6,7,8,9};
+unsigned char BImageViewTest::data[] = {1,2,3,4,5,6,7,8,9};
 
-typedef ImageTest<vigra::DImage> DImageTest;
+typedef ImageViewTest<vigra::DImage> DImageViewTest;
 
 template <>
-double DImageTest::data[] = {1.1,2.2,3.3,4.4,5.5,6.6,7.7,8.8,9.9};
+double DImageViewTest::data[] = {1.1,2.2,3.3,4.4,5.5,6.6,7.7,8.8,9.9};
 
-typedef ImageTest<vigra::BRGBImage> BRGBImageTest;
+typedef ImageViewTest<vigra::BRGBImage> BRGBImageViewTest;
 typedef vigra::RGBValue<unsigned char> BRGB;
 template <>
-BRGB BRGBImageTest::data[] = {
+BRGB BRGBImageViewTest::data[] = {
     BRGB(1,1,1),
     BRGB(2,2,2),
     BRGB(3,3,3),
@@ -997,40 +1063,509 @@ BRGB BRGBImageTest::data[] = {
     BRGB(9,9,9)
 };
 
-typedef ImageTest<vigra::FRGBImage> FRGBImageTest;
+typedef ImageViewTest<vigra::FRGBImage> FRGBImageViewTest;
 typedef vigra::RGBValue<float> FRGB;
 template <>
-FRGB FRGBImageTest::data[] = {
-    FRGB(1.1, 1.1, 1.1),
-    FRGB(2.2, 2.2, 2.2),
-    FRGB(3.3, 3.3, 3.3),
-    FRGB(4.4, 4.4, 4.4),
-    FRGB(5.5, 5.5, 5.5),
-    FRGB(6.6, 6.6, 6.6),
-    FRGB(7.7, 7.7, 7.7),
-    FRGB(8.8, 8.8, 8.8),
-    FRGB(9.9, 9.9, 9.9)
+FRGB FRGBImageViewTest::data[] = {
+    FRGB(1.1f, 1.1f, 1.1f),
+    FRGB(2.2f, 2.2f, 2.2f),
+    FRGB(3.3f, 3.3f, 3.3f),
+    FRGB(4.4f, 4.4f, 4.4f),
+    FRGB(5.5f, 5.5f, 5.5f),
+    FRGB(6.6f, 6.6f, 6.6f),
+    FRGB(7.7f, 7.7f, 7.7f),
+    FRGB(8.8f, 8.8f, 8.8f),
+    FRGB(9.9f, 9.9f, 9.9f)
+};
+
+struct MultiArrayPointoperatorsTest
+{
+
+    typedef float PixelType;
+    typedef MultiArray<3,PixelType> Image3D;
+    typedef MultiArrayView<3,PixelType> View3D;
+    typedef Image3D::difference_type Size3;
+    typedef MultiArray<1,PixelType> Image1D;
+    typedef Image1D::difference_type Size1;
+
+    Image3D img;
+
+    MultiArrayPointoperatorsTest()
+    : img(Size3(5,4,3))
+    {
+        int i;
+        PixelType c = 0.1f;
+        for(i=0; i<img.elementCount(); ++i, ++c)
+            img.data()[i] = c;
+    }
+
+    void testInit()
+    {
+        Image3D res(img.shape());
+        const Image3D::value_type ini = 1.1f;
+        should(res.shape() == Size3(5,4,3));
+
+        initMultiArray(destMultiArrayRange(res), ini);
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), ini);
+    }
+
+    void testCopy()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        copyMultiArray(srcMultiArrayRange(img), destMultiArray(res));
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), img(x,y,z));
+    }
+
+    void testCopyOuterExpansion()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        View3D view = img.subarray(Size3(0,0,0), Size3(5,1,1));
+        
+        copyMultiArray(srcMultiArrayRange(view), destMultiArrayRange(res));
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), img(x,0,0));
+    }
+
+    void testCopyInnerExpansion()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        View3D view = img.subarray(Size3(0,0,0), Size3(1,1,3));
+        
+        copyMultiArray(srcMultiArrayRange(view), destMultiArrayRange(res));
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), img(0,0,z));
+    }
+
+    void testTransform()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        transformMultiArray(srcMultiArrayRange(img), destMultiArray(res),
+                            Arg1() + Arg1());
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 2.0*img(x,y,z));
+    }
+
+    void testTransformOuterExpand()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        View3D view = img.subarray(Size3(0,0,0), Size3(5,1,1));
+        
+        transformMultiArray(srcMultiArrayRange(view), destMultiArrayRange(res),
+                            Arg1() + Arg1());
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 2.0*img(x,0,0));
+    }
+
+    void testTransformInnerExpand()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        View3D view = img.subarray(Size3(0,0,0), Size3(1,1,3));
+        
+        transformMultiArray(srcMultiArrayRange(view), destMultiArrayRange(res),
+                            Arg1() + Arg1());
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 2.0*img(0,0,z));
+    }
+
+    void testTransformOuterReduce()
+    {
+        Image3D res(Size3(5,1,1));
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        transformMultiArray(srcMultiArrayRange(img), destMultiArrayRange(res),
+                            reduceFunctor(Arg1() + Arg2(), 0.0));
+        
+        int x,y,z;
+        for(x=0; x<img.shape(0); ++x)
+        {
+            double sum = 0.0;
+            for(y=0; y<img.shape(1); ++y)
+                for(z=0; z<img.shape(2); ++z)
+                    sum += img(x,y,z);
+            shouldEqual(res(x,0,0), sum);
+        }
+        
+        Image1D res1(Size1(5));
+        MultiArrayView<3,PixelType> res3 = res1.insertSingletonDimension(1).insertSingletonDimension(2);
+        transformMultiArray(srcMultiArrayRange(img), 
+                            destMultiArrayRange(res3),
+                            FindSum<PixelType>());
+        shouldEqualSequenceTolerance(res1.data(), res1.data()+5, res.data(), 1e-7);       
+    }
+
+    void testTransformInnerReduce()
+    {
+        Image3D res(Size3(1,1,3));
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        transformMultiArray(srcMultiArrayRange(img), destMultiArrayRange(res),
+                            reduceFunctor(Arg1() + Arg2(), 0.0));
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+        {
+            double sum = 0.0;
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    sum += img(x,y,z);
+            shouldEqual(res(0,0,z), sum);
+        }
+        
+        Image1D res1(Size1(3));
+        MultiArrayView<3,PixelType> res3 = res1.insertSingletonDimension(0).insertSingletonDimension(0);
+        transformMultiArray(srcMultiArrayRange(img), 
+                            destMultiArrayRange(res3),
+                            FindSum<PixelType>());
+        shouldEqualSequenceTolerance(res1.data(), res1.data()+3, res.data(), 1e-6);       
+    }
+
+    void testCombine2()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        combineTwoMultiArrays(srcMultiArrayRange(img), srcMultiArray(img), 
+                              destMultiArray(res),
+                              Arg1() + Arg2());
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 2.0*img(x,y,z));
+    }
+
+    void testCombine2OuterExpand()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        View3D view = img.subarray(Size3(0,0,0), Size3(5,1,1));
+        combineTwoMultiArrays(srcMultiArrayRange(view), srcMultiArrayRange(img), 
+                              destMultiArrayRange(res),
+                              Arg1() + Param(2.0)*Arg2());       
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 2.0*img(x,y,z) + img(x,0,0));
+
+        combineTwoMultiArrays(srcMultiArrayRange(img), srcMultiArrayRange(view), 
+                              destMultiArrayRange(res),
+                              Arg1() + Param(2.0)*Arg2());       
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), img(x,y,z) + 2.0*img(x,0,0));
+
+        combineTwoMultiArrays(srcMultiArrayRange(view), srcMultiArrayRange(view), 
+                              destMultiArrayRange(res),
+                              Arg1() + Param(2.0)*Arg2());       
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 3.0*img(x,0,0));
+    }
+
+    void testCombine2InnerExpand()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        View3D view = img.subarray(Size3(0,0,0), Size3(1,1,3));
+        combineTwoMultiArrays(srcMultiArrayRange(view), srcMultiArrayRange(img), 
+                              destMultiArrayRange(res),
+                              Arg1() + Param(2.0)*Arg2());       
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 2.0*img(x,y,z) + img(0,0,z));
+
+        combineTwoMultiArrays(srcMultiArrayRange(img), srcMultiArrayRange(view), 
+                              destMultiArrayRange(res),
+                              Arg1() + Param(2.0)*Arg2());       
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), img(x,y,z) + 2.0*img(0,0,z));
+
+        combineTwoMultiArrays(srcMultiArrayRange(view), srcMultiArrayRange(view), 
+                              destMultiArrayRange(res),
+                              Arg1() + Param(2.0)*Arg2());       
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 3.0*img(0,0,z));
+    }
+
+    void testCombine2OuterReduce()
+    {
+        Image3D res(Size3(5,1,1));
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        combineTwoMultiArrays(srcMultiArrayRange(img), srcMultiArrayRange(img), 
+                              destMultiArrayRange(res),
+                              reduceFunctor(Arg1() + Arg2() + Arg3(), 0.0));
+        
+        int x,y,z;
+        for(x=0; x<img.shape(0); ++x)
+        {
+            double sum = 0.0;
+            for(y=0; y<img.shape(1); ++y)
+                for(z=0; z<img.shape(2); ++z)
+                    sum += img(x,y,z);
+            shouldEqual(res(x,0,0), 2.0*sum);
+        }
+    }
+
+    void testCombine2InnerReduce()
+    {
+        Image3D res(Size3(1,1,3));
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        combineTwoMultiArrays(srcMultiArrayRange(img), srcMultiArrayRange(img), 
+                              destMultiArrayRange(res),
+                              reduceFunctor(Arg1() + Arg2() + Arg3(), 0.0));
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+        {
+            double sum = 0.0;
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    sum += img(x,y,z);
+            shouldEqual(res(0,0,z), 2.0*sum);
+        }
+    }
+
+    void testCombine3()
+    {
+        Image3D res(img.shape());
+        
+        initMultiArray(destMultiArrayRange(res), 0.0);
+        
+        combineThreeMultiArrays(srcMultiArrayRange(img), 
+                                srcMultiArray(img), srcMultiArray(img), 
+                                destMultiArray(res),
+                                Arg1() + Arg2() + Arg3());
+        
+        int x,y,z;
+        for(z=0; z<img.shape(2); ++z)
+            for(y=0; y<img.shape(1); ++y)
+                for(x=0; x<img.shape(0); ++x)
+                    shouldEqual(res(x,y,z), 3.0*img(x,y,z));
+    }
+    
+    void testInitMultiArrayBorder(){
+        typedef vigra::MultiArray<1,int> IntLine;
+        typedef vigra::MultiArray<2,int> IntImage;
+        typedef vigra::MultiArray<3,int> IntVolume;
+        
+        const int desired_vol[] ={  0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0};
+
+        const int desired_img[] ={  0, 0, 0, 0, 0, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 0, 0, 0, 0, 0};
+
+        const int desired_lin[] ={  0, 0, 0, 5, 0, 0, 0 };
+
+        const int desired_vol2[] ={  0, 0,
+                                     0, 0,
+
+                                     0, 0, 
+                                     0, 0};
+
+        IntVolume vol(IntVolume::difference_type(6,6,6));
+        
+        for(IntVolume::iterator iter=vol.begin(); iter!=vol.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(vol),2,0);
+        shouldEqualSequence(vol.begin(), vol.end(), desired_vol);
+
+        IntImage img(IntImage::difference_type(6,6));
+        
+        for(IntImage::iterator iter=img.begin(); iter!=img.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(img),1,0);
+        shouldEqualSequence(img.begin(), img.end(), desired_img);
+
+        IntLine lin(IntLine::difference_type(7));
+        
+        for(IntLine::iterator iter=lin.begin(); iter!=lin.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(lin),3,0);
+        shouldEqualSequence(lin.begin(), lin.end(), desired_lin);
+
+        IntVolume vol2(IntVolume::difference_type(2,2,2));
+        
+        for(IntVolume::iterator iter=vol2.begin(); iter!=vol2.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(vol2),9,0);
+        shouldEqualSequence(vol2.begin(), vol2.end(), desired_vol2);
+
+    }
+    
+    void testTensorUtilities()
+    {
+        MultiArrayShape<2>::type shape(3,4);
+        int size = shape[0]*shape[1];
+        
+        MultiArray<2, TinyVector<double, 2> > vector(shape), rvector(shape);
+        MultiArray<2, TinyVector<double, 3> > tensor1(shape), tensor2(shape), rtensor(shape);
+        MultiArray<2, double > trace(shape), rtrace(shape);
+        MultiArray<2, double > determinant(shape), rdet(shape);
+        
+        for(int k=0; k<size; ++k)
+        {
+            for(int l=0; l<2; ++l)
+                vector[k][l] = randomMT19937().uniform();
+            for(int l=0; l<3; ++l)
+                tensor1[k][l] = randomMT19937().uniform();
+            rdet[k] = tensor1[k][0]*tensor1[k][2] - sq(tensor1[k][1]);
+        }
+        
+        vectorToTensor(srcImageRange(vector), destImage(rtensor));
+        vectorToTensorMultiArray(srcMultiArrayRange(vector), destMultiArray(tensor2));
+        shouldEqualSequence(tensor2.data(), tensor2.data()+size, rtensor.data());
+                
+        tensorTrace(srcImageRange(tensor1), destImage(rtrace));
+        tensorTraceMultiArray(srcMultiArrayRange(tensor1), destMultiArray(trace));
+        shouldEqualSequence(trace.data(), trace.data()+size, rtrace.data());
+                
+        tensorDeterminantMultiArray(srcMultiArrayRange(tensor1), destMultiArray(determinant));
+        shouldEqualSequence(determinant.data(), determinant.data()+size, rdet.data());
+                
+        tensorDeterminantMultiArray(srcMultiArrayRange(tensor2), destMultiArray(determinant));
+        shouldEqualTolerance(norm(determinant), 0.0, 1e-14);
+
+        tensorEigenRepresentation(srcImageRange(tensor1), destImage(rtensor));
+        tensorEigenvaluesMultiArray(srcMultiArrayRange(tensor1), destMultiArray(vector));
+        for(int k=0; k<size; ++k)
+        {
+            shouldEqualTolerance(vector[k][0], rtensor[k][0], 1e-14);
+            shouldEqualTolerance(vector[k][1], rtensor[k][1], 1e-14);
+        }
+    }
 };
 
 
-struct ImageTestSuite
+struct ImageViewTestSuite
 : public vigra::test_suite
 {
-    ImageTestSuite()
-    : vigra::test_suite("ImageTestSuite")
+    ImageViewTestSuite()
+    : vigra::test_suite("ImageViewTestSuite")
     {
-        add( testCase( &BImageTest::testBasicImageIterator));
-        add( testCase( &BImageTest::testImageIterator));
-        add( testCase( &BImageTest::copyImage));
-        add( testCase( &DImageTest::testBasicImageIterator));
-        add( testCase( &DImageTest::testImageIterator));
-        add( testCase( &DImageTest::copyImage));
-        add( testCase( &BRGBImageTest::testBasicImageIterator));
-        add( testCase( &BRGBImageTest::testImageIterator));
-        add( testCase( &BRGBImageTest::copyImage));
-        add( testCase( &FRGBImageTest::testBasicImageIterator));
-        add( testCase( &FRGBImageTest::testImageIterator));
-        add( testCase( &FRGBImageTest::copyImage));
+        add( testCase( &BImageViewTest::testBasicImageIterator));
+        add( testCase( &BImageViewTest::testImageIterator));
+        add( testCase( &BImageViewTest::copyImage));
+        add( testCase( &DImageViewTest::testBasicImageIterator));
+        add( testCase( &DImageViewTest::testImageIterator));
+        add( testCase( &DImageViewTest::copyImage));
+        add( testCase( &BRGBImageViewTest::testBasicImageIterator));
+        add( testCase( &BRGBImageViewTest::testImageIterator));
+        add( testCase( &BRGBImageViewTest::copyImage));
+        add( testCase( &FRGBImageViewTest::testBasicImageIterator));
+        add( testCase( &FRGBImageViewTest::testImageIterator));
+        add( testCase( &FRGBImageViewTest::copyImage));
     }
 };
 
@@ -1056,8 +1591,10 @@ struct MultiArrayTestSuite
         add( testCase( &MultiArrayTest::test_reshape) );
         add( testCase( &MultiArrayTest::test_subarray ) );
         add( testCase( &MultiArrayTest::test_stridearray ) );
+        add( testCase( &MultiArrayTest::test_copy_int_float ) );
+
         add( testCase( &MultiImpexTest::testImpex ) );
-    }
+	}
 };
 
 struct MultiArrayDataTestSuite
@@ -1066,6 +1603,7 @@ struct MultiArrayDataTestSuite
     MultiArrayDataTestSuite()
     : vigra::test_suite("MultiArrayDataTestSuite")
     {
+        add( testCase( &MultiArrayDataTest::testHasData ) );
         add( testCase( &MultiArrayDataTest::testEquality ) );
         add( testCase( &MultiArrayDataTest::test_subarray ) );
         add( testCase( &MultiArrayDataTest::test_stridearray ) );
@@ -1074,6 +1612,7 @@ struct MultiArrayDataTestSuite
         add( testCase( &MultiArrayDataTest::test_bindAt ) );
         add( testCase( &MultiArrayDataTest::test_bind ) );
         add( testCase( &MultiArrayDataTest::test_bind0 ) );
+        add( testCase( &MultiArrayDataTest::test_singletonDimension ) );
         add( testCase( &MultiArrayDataTest::testPermute ) );
         add( testCase( &MultiArrayDataTest::testNorm ) );
         add( testCase( &MultiArrayDataTest::testScanOrderAccess ) );
@@ -1082,24 +1621,55 @@ struct MultiArrayDataTestSuite
     }
 };
 
+struct MultiArrayPointOperatorsTestSuite
+: public vigra::test_suite
+{
+  MultiArrayPointOperatorsTestSuite()
+    : vigra::test_suite("MultiArrayPointOperatorsTestSuite")
+    {
+        add( testCase( &MultiArrayPointoperatorsTest::testInit ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCopy ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCopyOuterExpansion ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCopyInnerExpansion ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testTransform ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testTransformOuterExpand ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testTransformInnerExpand ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testTransformOuterReduce ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testTransformInnerReduce ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCombine2 ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCombine2OuterExpand ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCombine2InnerExpand ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCombine2OuterReduce ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCombine2InnerReduce ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testCombine3 ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testInitMultiArrayBorder ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testTensorUtilities ) );
+    }
+}; // struct MultiArrayPointOperatorsTestSuite
 
-int main()
+
+int main(int argc, char ** argv)
 {
     // run the multi-array testsuite
     MultiArrayTestSuite test1;
-    int failed = test1.run();
+    int failed = test1.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test1.report() << std::endl;
 
     // run the multi-array data-testsuite
     MultiArrayDataTestSuite test1a;
-    failed += test1a.run();
+    failed += test1a.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test1a.report() << std::endl;
 
     // run the image testsuite
-    ImageTestSuite test2;
-    failed += test2.run();
+    ImageViewTestSuite test2;
+    failed += test2.run(vigra::testsToBeExecuted(argc, argv));
     std::cout << test2.report() << std::endl;
 
+    // run the multi-array point operator test suite
+    MultiArrayPointOperatorsTestSuite test3;
+    failed += test3.run(vigra::testsToBeExecuted(argc, argv));
+    std::cout << test3.report() << std::endl;
+    
     return (failed != 0);
 }
 
