@@ -1,11 +1,10 @@
 /************************************************************************/
 /*                                                                      */
 /*                 Copyright 2004 by Ullrich Koethe                     */
-/*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
 /*    The VIGRA Website is                                              */
-/*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
+/*        http://hci.iwr.uni-heidelberg.de/vigra/                       */
 /*    Please direct questions, bug reports, and contributions to        */
 /*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
 /*        vigra@informatik.uni-hamburg.de                               */
@@ -49,11 +48,14 @@
 #include "vigra/symmetry.hxx"
 #include "vigra/watersheds.hxx"
 #include "vigra/noise_normalization.hxx"
-#include "vigra/slanted_edge_mtf.hxx"
 #include "vigra/affinegeometry.hxx"
 #include "vigra/affine_registration.hxx"
 #include "vigra/impex.hxx"
 
+#ifdef HasFFTW3
+# include "vigra/slanted_edge_mtf.hxx"
+#endif
+
 using namespace vigra;
 
 struct LabelingTest
@@ -584,8 +586,6 @@ struct EdgeDetectionTest
     {
         std::vector<vigra::Edgel> edgels;
         cannyEdgelList(srcImageRange(imgCanny), edgels, 1.0);
-        std::vector<vigra::Edgel> edgels3x3;
-        cannyEdgelList3x3(srcImageRange(imgCanny), edgels3x3, 1.0);
         int count = 0;
         for(unsigned int i=0; i<edgels.size(); ++i)
         {
@@ -593,7 +593,7 @@ struct EdgeDetectionTest
                 continue;  // ignore edgels that result from round off error during convolution
             ++count;
             should(edgels[i].x == edgels[i].y);
-            should(VIGRA_CSTD::fabs(edgels[i].orientation-M_PI*0.75) < 0.1);
+            should(VIGRA_CSTD::fabs(edgels[i].orientation-M_PI*0.25) < 0.1);
         }
         should(count == 75);
     }
@@ -609,7 +609,7 @@ struct EdgeDetectionTest
                 continue;  // ignore edgels that result from round off error during convolution
             ++count;
             should(edgels[i].x == edgels[i].y);
-            should(VIGRA_CSTD::fabs(edgels[i].orientation-M_PI*0.75) < 0.1);
+            should(VIGRA_CSTD::fabs(edgels[i].orientation-M_PI*0.25) < 0.1);
         }
         should(count == 38);
     }
@@ -1532,6 +1532,7 @@ struct NoiseNormalizationTest
     }
 };
 
+#ifdef HasFFTW3
 struct SlantedEdgeMTFTest
 {
     typedef vigra::DImage Image;
@@ -1583,6 +1584,7 @@ struct SlantedEdgeMTFTest
         shouldEqualTolerance(mtfFitGaussian(res), 0.5, 1e-2);
     }
 };
+#endif // HasFFTW3
 
 struct AffineRegistrationTest
 {
@@ -1731,19 +1733,21 @@ struct SimpleAnalysisTestSuite
         add( testCase( &NoiseNormalizationTest::testNonparametricNoiseNormalizationU8));
         add( testCase( &NoiseNormalizationTest::testParametricNoiseNormalizationRGB));
         add( testCase( &NoiseNormalizationTest::testNonparametricNoiseNormalizationRGB));
-        add( testCase( &SlantedEdgeMTFTest::testSlantedEdgeMTF));
         add( testCase( &AffineRegistrationTest::testCorrespondingPoints));
         add( testCase( &AffineRegistrationTest::testTranslationRegistration));
         add( testCase( &AffineRegistrationTest::testSimilarityRegistration));
         add( testCase( &AffineRegistrationTest::testAffineRegistration));
+#ifdef HasFFTW3
+        add( testCase( &SlantedEdgeMTFTest::testSlantedEdgeMTF));
+#endif // HasFFTW3
     }
 };
 
-int main()
+int main(int argc, char ** argv)
 {
     SimpleAnalysisTestSuite test;
 
-    int failed = test.run();
+    int failed = test.run(vigra::testsToBeExecuted(argc, argv));
 
     std::cout << test.report() << std::endl;
 
